@@ -4,15 +4,17 @@ use warnings qw(FATAL all NONFATAL misc);
 use 5.010; no if $] >= 5.017011, warnings => "experimental";
 
 use Carp qw(carp croak confess);
-our $VERSION = '0.101';
-#use mro 'c3';
+our $VERSION = '0.110';
+use mro 'c3';
 
 use Scalar::Util qw/weaken/;
+use List::MoreUtils qw/uniq/;
 
 #
 # YATT Internalへの Facade. YATT の初期化パラメータの保持者でもある。
 #
 use parent qw/YATT::Lite::Object File::Spec/;
+use YATT::Lite::Partial::MarkAfterNew -as_base;
 use YATT::Lite::MFields qw/YATT
 	      cf_dir
 	      cf_vfs cf_base
@@ -23,7 +25,10 @@ use YATT::Lite::MFields qw/YATT
 	      cf_index_name
 	      cf_ext_public
 	      cf_ext_private
-	      cf_app_ns entns
+	      cf_app_ns
+	      entns
+	      cgen_class
+
 	      cf_app_name
 	      cf_debug_cgen cf_debug_parser cf_namespace cf_only_parse
 	      cf_special_entities cf_no_lineinfo cf_check_lineno
@@ -34,18 +39,28 @@ use YATT::Lite::MFields qw/YATT
 	      cf_info
 	      cf_lcmsg_sink
 	      cf_always_refresh_deps
+	      cf_no_mro_c3
+	      cf_render_as_bytes
 
 	      cf_default_lang
 
-	      cf_path2entns
+	      cf_entns2vfs_item
+	      cf_import
+              cf_match_argsroute_first
+              cf_stash_unknown_params_to
+              cf_body_argument
+              cf_body_argument_type
+	      cf_prefer_call_for_entity
 	    /;
 
+use constant DEBUG => $ENV{DEBUG_YATT_LITE};
+
 MY->cf_mkaccessors(qw/app_name/);
 
 # Entities を多重継承する理由は import も継承したいから。
 # XXX: やっぱり、 YATT::Lite には固有の import を用意すべきではないか?
 #   yatt_default や cgen_perl を定義するための。
-use YATT::Lite::Entities -as_base, qw(*YATT *CON *SYS);
+use YATT::Lite::Entities -as_base, qw(*YATT *CON *SYS import);
 
 # For error, raise, DONE. This is inserted to ISA too.
 use YATT::Lite::Partial::ErrorReporter;
@@ -55,21 +70,27 @@ use YATT::Lite::Partial::AppPath;
 use YATT::Lite::Util qw/globref lexpand extname ckrequire terse_dump escape
 			set_inc ostream try_invoke list_isa symtab
 			look_for_globref
-			subname ckeval
+			subname ckeval ckrequire
 			secure_text_plain
+                        raise_psgi_error
 			define_const
+                        add_entity_into
 		       /;
 
 sub Facade () {__PACKAGE__}
-sub default_app_ns {'MyApp'}
+sub default_app_ns {'MyYATT'}
 sub default_trans {'YATT::Lite::Core'}
 sub default_export {(shift->SUPER::default_export, qw(Entity *SYS *CON))}
 sub default_index_name { '' }
 sub default_ext_public {'yatt'}
 sub default_ext_private {'ytmpl'}
+sub default_body_argument { 'body' }
+sub default_body_argument_type { 'code' }
+sub default_output_encoding { 'utf-8' }
 
 sub with_system {
   (my MY $self, local $SYS, my $method) = splice @_, 0, 3;
+  local $YATT = $self;
   $self->$method(@_);
 }
 
@@ -79,14 +100,80 @@ sub after_new {
   $self->{cf_index_name} //= "";
   $self->{cf_ext_public} //= $self->default_ext_public;
   $self->{cf_ext_private} //= $self->default_ext_private;
+  $self->{cf_body_argument} //= $self->default_body_argument;
+  $self->{cf_body_argument_type} //= $self->default_body_argument_type;
+  $self->{cf_output_encoding} //= $self->default_output_encoding;
+}
+
+sub _after_after_new {
+  (my MY $self) = @_;
   weaken($self->{cf_factory});
 }
 
+sub rel_app_name {
+  (my MY $self) = @_;
+  unless ($SYS) {
+    Carp::croak "rel_app_name() is called without setting \$SYS!";
+  }
+  # XXX: This assumes $dir is somewhere under $app_root tree.
+  my $rel_app_name = substr($self->{cf_dir}, length($SYS->app_root));
+  # Make sure leading / is removed.
+  $rel_app_name =~ s|^/||;
+  $rel_app_name;
+}
+
 # XXX: kludge!
-sub create_neighbor {
+sub find_neighbor_yatt {
   (my MY $self, my ($dir)) = @_;
-  my MY $yatt = $self->{cf_factory}->load_yatt($dir);
-  $yatt->get_trans->root;
+  $self->{cf_factory}->load_yatt($dir);
+}
+sub find_neighbor_vfs {
+  (my MY $self, my ($dir)) = @_;
+  $self->find_neighbor_yatt($dir)->get_trans;
+}
+sub find_neighbor {
+  (my MY $self, my ($dir)) = @_;
+  $self->find_neighbor_vfs($dir)->root;
+}
+
+#
+# list all configs (named $name). (base first, then local one)
+# (useful to avoid config repeation)
+#
+sub cget_all {
+  (my MY $self, my $name) = @_;
+  (map($_->cget_all($name)
+       , $self->list_base_obj)
+   , lexpand($self->{"cf_$name"}));
+}
+
+sub list_base_obj {
+  (my MY $self) = @_;
+  map {
+    $self->find_neighbor_yatt($self->app_path_normalize($_))
+  } $self->list_base_dir;
+}
+
+sub list_base_dir {
+  (my MY $self) = @_;
+
+  my $base = $self->{cf_base} // do {
+    my %vfs = lexpand($self->{cf_vfs});
+    [map {
+      #
+      # Each element of $vfs{base} is either ARRAY (of vfs spec)
+      # or YATT::Lite::VFS::Dir object (instantiated from spec).
+      #
+      if (ref $_ eq 'ARRAY') {
+	my %vfs_base = @$_;
+	$vfs_base{dir};
+      } else {
+	$_->{cf_path};
+      }
+    } lexpand($vfs{base})];
+  };
+
+  lexpand($base);
 }
 
 #========================================
@@ -107,28 +194,58 @@ sub handle {
   }
 
   my $sub = $YATT->find_handler($ext, $file, $CON);
-  $sub->($YATT, $CON, $file);
+
+  my $result = $sub->($YATT, $CON, $file);
 
   try_invoke($CON, 'flush_headers');
 
-  $CON;
+  $result;
 }
 
 sub render {
+  my MY $self = shift;
+  my $raw_bytes = $self->render_encoded(@_);
+  if ($self->{cf_render_as_bytes}) {
+    $raw_bytes
+  } else {
+    Encode::decode(utf8 => $raw_bytes);
+  }
+}
+
+sub render_encoded {
   my MY $self = shift;
   my $buffer; {
+    my $encName = $self->{cf_render_as_bytes} ? undef : $self->{cf_output_encoding};
     my $con = $SYS
-      ? $SYS->make_connection(undef, buffer => \$buffer, yatt => $self)
-	: ostream(\$buffer);
+      ? $SYS->make_connection(undef, buffer => \$buffer, yatt => $self,
+                              noheader => 1,
+                              encoding => $encName)
+	: ostream(\$buffer, $encName ? ":encoding($encName)" : "");
     $self->render_into($con, @_);
   }
   $buffer;
 }
 
 sub render_into {
-  local ($YATT, $CON) = splice @_, 0, 2;
-  $YATT->open_trans->render_into($CON, @_);
-  try_invoke($CON, 'flush_headers');
+  (my MY $self, my ($con, $file, $args)) = @_;
+  local ($YATT, $CON) = ($self, $con);
+  if (ref $file eq 'ARRAY') {
+    # [$file, $type, $item]
+    $self->open_trans->render_into($CON, $file, $args);
+  } else {
+    $self->raw_render_into(
+      $con,
+      YATT::Lite::Util::rootname($file || $self->{cf_index_name}),
+      $args,
+    );
+  }
+  try_invoke($con, 'flush_headers');
+}
+
+sub raw_render_into {
+  (local ($YATT), my ($con, $file, $args)) = @_;
+  my ($part, $sub, $pkg) = $YATT->open_trans->find_part_renderer($file);
+  $sub->($pkg, $con, $part->reorder_hash_params($args));
 }
 
 sub find_handler {
@@ -156,8 +273,8 @@ sub _handle_yatt {
 
 sub _handle_ytmpl {
   (my MY $self, my ($con, $file)) = @_;
-  # XXX: http result code:
-  print $con "Forbidden filetype: $file";
+
+  $self->raise_psgi_error(403, "Forbidden filetype: $file");
 }
 
 #----------------------------------------
@@ -167,7 +284,7 @@ sub prepare_part_handler {
 
   my $trans = $self->open_trans;
 
-  my $mapped = [$file, my ($type, $item) = $self->parse_request_sigil($con)];
+  my $mapped = [$file, my ($type, $item) = $con->sigil_type_item()];
   if (not $self->{cf_dont_debug_param}
       and -e ".htdebug_param") {
     $self->dump($mapped, [map {[$_ => $con->param($_)]} $con->param]);
@@ -180,54 +297,16 @@ sub prepare_part_handler {
     croak $self->error(q|Forbidden request %s|, terse_dump($mapped));
   }
 
-  my @args; @args = $part->reorder_cgi_params($con)
+  my @args; @args = $trans->reorder_cgi_params($part, $con)
     unless $self->{cf_dont_map_args} || $part->isa($trans->Action);
 
   ($part, $sub, $pkg, \@args);
 }
 
+# XXX: deprecated
 sub parse_request_sigil {
   (my MY $self, my ($con)) = @_;
-  my ($subpage, $action);
-  # XXX: url_param
-  foreach my $name (grep {defined} $con->param()) {
-    my ($sigil, $word) = $name =~ /^([~!])(\1|\w*)$/
-      or next;
-    # If $name in ('~~', '!!'), use value.
-    my $new = $word eq $sigil ? $con->param($name) : $word;
-    # else use $word from ~$word.
-    # Note: $word may eq ''. This is for render_/action_.
-    given ($sigil) {
-      when ('~') {
-	if (defined $subpage) {
-	  $self->error("Duplicate subpage request! %s vs %s"
-		       , $subpage, $new);
-	}
-	$subpage = $new;
-      }
-      when ('!') {
-	if (defined $action) {
-	  $self->error("Duplicate action! %s vs %s"
-		       , $action, $new);
-	}
-	$action = $new;
-      }
-      default {
-	croak "Really?";
-      }
-    }
-  }
-  if (defined $subpage and defined $action) {
-    # XXX: Reserved for future use.
-    $self->error("Can't use subpage and action at one time: %s vs %s"
-		 , $subpage, $action);
-  } elsif (defined $subpage) {
-    (page => $subpage);
-  } elsif (defined $action) {
-    (action => $action);
-  } else {
-    ();
-  }
+  $con->sigil_type_item();
 }
 
 sub cut_ext {
@@ -287,21 +366,37 @@ sub build_trans {
     (\@vfsspec
      , facade => $self
      , cache => $vfscache
+     , entns2vfs_item => $self->{cf_entns2vfs_item}
      , entns => $self->{entns}
      , @rest
-     # XXX: Should be more extensible.
-     , $self->cf_delegate_defined(qw/namespace base
-				     die_in_error tmpl_encoding
-				     debug_cgen debug_parser
-				     special_entities no_lineinfo check_lineno
-				     index_name
-				     ext_public
-				     ext_private
-				     rc_script
-				     lcmsg_sink
-				     only_parse
-				     always_refresh_deps
-				    /));
+     , $self->cf_delegate_defined($self->_cf_delegates));
+}
+
+sub _cf_delegates {
+  qw(namespace
+     base
+     die_in_error
+     tmpl_encoding
+     debug_cgen
+     debug_parser
+     special_entities
+     no_lineinfo
+     check_lineno
+     index_name
+     ext_public
+     ext_private
+     rc_script
+     lcmsg_sink
+     only_parse
+     always_refresh_deps
+     no_mro_c3
+     import
+     match_argsroute_first
+     stash_unknown_params_to
+     body_argument
+     body_argument_type
+     prefer_call_for_entity
+    )
 }
 
 sub _before_after_new {
@@ -310,6 +405,31 @@ sub _before_after_new {
   $self->{entns} = $self->ensure_entns($self->{cf_app_ns});
 }
 
+#========================================
+# Code generator class
+#========================================
+
+sub root_CGEN_perl () { 'YATT::Lite::CGen::Perl' }
+*CGEN_perl = *root_CGEN_perl; *CGEN_perl = *root_CGEN_perl;
+sub ensure_cgen_for {
+  my ($mypack, $type, $app_ns) = @_;
+  $mypack->ensure_supplns("CGEN_$type" => $app_ns);
+}
+
+sub get_cgen_class {
+  (my MY $self, my $type) = @_;
+  my $name = "CGEN_$type";
+  my $sub = $self->can("root_$name")
+    or croak "Unknown cgen class: $type";
+  $self->{cgen_class}{$type}
+    ||= $self->ensure_cgen_for($type, $self->{cf_app_ns});
+}
+
+sub is_default_cgen_ready {
+  (my MY $self) = @_;
+  $self->{cgen_class}{perl};
+}
+
 #========================================
 # Entity
 #========================================
@@ -320,45 +440,120 @@ sub root_EntNS { 'YATT::Lite::Entities' }
 # $app_ns に EntNS constant を追加する。
 # 複数回呼ばれた場合、既に定義済みの entns を返す
 
-sub ensure_entns {
-  my ($mypack, $app_ns, @baseclass) = @_;
-  my $entns = "${app_ns}::EntNS";
+sub should_use_mro_c3 {
+  (my MY $self_or_pack) = @_;
+  if (ref $self_or_pack) {
+    not $self_or_pack->{cf_no_mro_c3}
+  } else {
+    mro::get_mro($self_or_pack) eq 'c3';
+  }
+}
 
-  my $sym = do {no strict 'refs'; \*{$entns}};
+#========================================
+
+# These ns-related methods (ensure_...) are called as Class Methods.
+# This means you can't touch instance fields.
+
+# Old interface.
+# ensure_entns($app_ns, @base_entns)
+# returns EntNS for $app_ns with correct inheritance settings.
+#
+sub ensure_entns {
+  my ($mypack, $app_ns, @base_entns) = @_;
+  my $entns = $mypack->ensure_supplns(EntNS => $app_ns, \@base_entns
+				      , undef, +{no_fields => 1});
+  $entns;
+}
+
+# New interface.
+# ensure_supplns($kind, $app_ns, [@base_suppls], [@base_mains], {%opts})
+# returns ${app_ns}::${kind} with correct inheritance.
+#
+# [@base_suppls] gives base supplemental classes for this supplns.
+# [@base_mains] gives (not supplemental but) main classes for this.
+#
+# If both base_suppls and base_mains is empty, base_mains is derived
+# from current @ISA of $app_ns.
+#
+sub ensure_supplns {
+  my ($mypack, $kind, $app_ns, $base_suppls, $base_mains, $opts) = @_;
+
+  my $supplns = join("::", $app_ns, $kind);
+
+  my $sym = do {no strict 'refs'; \*{$supplns}};
   if (*{$sym}{CODE}) {
-    # croak "EntNS for $app_ns is already defined!";
-    return $entns;
+    # croak "$kind for $app_ns is already defined!";
+    return $supplns;
   }
 
-  # mro::set_mro($entns, 'c3'); # XXX: Should change to c3, but...
+  my $app_ns_filename = do {
+    my $sub = $app_ns->can("filename");
+    $sub ? ("(For path '".($sub->() // '')."')") : "";
+  };
+
+  print STDERR "# First ensure_supplns $kind for $app_ns $app_ns_filename: "
+    , terse_dump($base_suppls, $base_mains, $opts), "\n" if DEBUG;
+
+  if (not $base_suppls and not $base_mains) {
+    my @isa = list_isa($app_ns);
+    print STDERR "# app_ns $app_ns isa: "
+      , terse_dump(@isa), "\n" if DEBUG;
+
+    $base_mains = $mypack->should_use_mro_c3
+      ? [reverse @isa] : \@isa;
+  }
+
+  my @baseclass = (lexpand($base_suppls), map {
+    $mypack->ensure_supplns($kind => $_, undef, undef, $opts);
+  } lexpand($base_mains));
+
+  if ($mypack->should_use_mro_c3) {
+    print STDERR "# $kind - Set mro c3 for $supplns $app_ns_filename since $mypack uses c3\n" if DEBUG;
+    mro::set_mro($supplns, 'c3')
+  } else {
+    print STDERR "# $kind - Keep mro dfs for $supplns $app_ns_filename since $mypack uses dfs\n" if DEBUG;
+  }
 
   # $app_ns が %FIELDS 定義を持たない時(ex YLObjectでもPartialでもない)に限り、
   # YATT::Lite への継承を設定する
   unless (YATT::Lite::MFields->has_fields($app_ns)) {
     # XXX: $mypack への継承にすると、あちこち動かなくなるぜ？なんで？
+    print STDERR "# app_ns - Add ISA for '$app_ns' with fields: ",MY,"\n"
+      if DEBUG;
     YATT::Lite::MFields->add_isa_to($app_ns, MY)->define_fields($app_ns);
   }
 
-  unless (grep {$_->can("EntNS")} @baseclass) {
-    my $base = try_invoke($app_ns, 'EntNS') // $mypack->root_EntNS;
-    # print "insert base '$base' for entns $entns\n";
-    unshift @baseclass, $base;
+  unless (grep {$_->can($kind)} @baseclass) {
+    my $base = try_invoke($app_ns, $kind) // $mypack->can("root_$kind")->();
+    ckrequire($base);
+    print STDERR "# $kind - Set default base for $supplns <- ($base)\n" if DEBUG;
+    if ($mypack->should_use_mro_c3) {
+      push @baseclass, $base;
+    } else {
+      unshift @baseclass, $base;
+    }
   }
 
-  # print "entns $entns should inherits: @baseclass\n";
-  YATT::Lite::MFields->add_isa_to($entns, @baseclass);
+  do {
+    my @cls = uniq @baseclass;
+    print STDERR "# $kind - Add ISA for $supplns <- (@cls)\n" if DEBUG;
+    YATT::Lite::MFields->add_isa_to($supplns, @cls);
+  };
+  if (not $opts->{no_fields}) {
+    YATT::Lite::MFields->define_fields($supplns);
+  }
 
-  set_inc($entns, 1);
+  set_inc($supplns, 1);
 
-  # EntNS() を足すのは最後にしないと、再帰継承に陥る
+  # $kind() を足すのは最後にしないと、再帰継承に陥る
   unless (my $code = *{$sym}{CODE}) {
-    define_const($sym, $entns);
-  } elsif ((my $old = $code->()) ne $entns) {
-    croak "Can't add EntNS() to '$app_ns'. Already has EntNS as $old!";
+    define_const($sym, $supplns);
+  } elsif ((my $old = $code->()) ne $supplns) {
+    croak "Can't add $kind() to '$app_ns'. Already has $kind as $old!";
   } else {
     # ok.
   }
-  $entns
+  $supplns
 }
 
 sub list_entns {
@@ -385,10 +580,9 @@ sub define_Entity {
   unless (*{$ent}{CODE}) {
     *$ent = sub {
       my ($name, $sub) = @_;
-      my $longname = join "::", $destns, "entity_$name";
-      subname($longname, $sub);
-      print "defining entity_$name in $destns\n" if $ENV{DEBUG_ENTNS};
-      *{globref($destns, "entity_$name")} = $sub;
+      if ($myPack->add_entity_into($destns, $name, $sub)) {
+        print STDERR "# defined entity $name in $destns\n" if DEBUG;
+      }
     };
   }
 
@@ -403,6 +597,58 @@ sub define_Entity {
   return $destns;
 }
 
+#
+# Note about 'Action' registration mechanism in .htyattrc.pl
+#
+#  First, *globref() = $action is not enough. Because...
+#
+#  There are 2 places to hold actions.
+#    1. $YATT->{Action}      <= comes from *.ydo
+#    2. $vfs_folder->{Item}  <= comes from !yatt:action in templates
+#
+#  Action in .htyattrc.pl is, special case of 2.
+#  Since 2. is managed by yatt vfs, it must be wrapped by Action object
+#  so that $vfs->find_part_handler works well.
+#
+#
+#  This is bit complicated because .htyattrc.pl is loaded *BEFORE* $YATT
+#  is instantiated. This means "Action => name, $handler" can not touch $YATT
+#  at that time. So, I need to delay actual registration until $YATT is created.
+#
+#  To achieve this, $handler is registered first in %Actions of caller,
+#  then installed into actual vfs.
+#
+# Also note: loading of .htyattrc.pl is handled by Factory.
+#
+sub ACTION_DICT_SYM () {'Actions'}
+sub define_Action {
+  my ($myPack, $opts, $callpack) = @_;
+
+  *{globref($callpack, ACTION_DICT_SYM)} = my $action_dict = +{};
+
+  *{globref($callpack, 'Action')} = sub {
+    my ($name, $sub) = @_;
+    my @caller = my ($callpack, $filename, $lineno) = caller;
+    if (defined (my $old = $action_dict->{$name})) {
+      croak "Duplicate definition of Action '$name'! previously"
+	." at $old->[1][1] line $old->[1][2]\n new at $filename line $lineno\n";
+    }
+    $action_dict->{$name} = [$sub, \@caller];
+  };
+}
+
+sub setup_rc_actions {
+  (my $self) = @_;
+  my $glob = look_for_globref($self, ACTION_DICT_SYM)
+    or return;
+  my $dict = *{$glob}{HASH};
+
+  my $vfs = $self->get_vfs;
+  foreach my $name (keys %$dict) {
+    $vfs->add_root_action_handler($name, @{$dict->{$name}});
+  }
+}
+
 # ここで言う Object系とは、
 #   YATT::Lite::Object を継承してるか、
 #   又は既に %FIELDS が定義されている class
@@ -458,9 +704,15 @@ sub default_lang {
 
 #========================================
 # Delegation to the core(Translator, which is useless for non-templating.)
+#
+# Note: these wrappers do not call YATT::Lite::VFS->reset_refresh_mark.
+# This means once templates is compiled, it is not updated even if
+# they are modified. You can avoid this behavior by
+# directly using $YATT->open_trans->xxx instead.
 #========================================
 foreach
   (qw/find_part
+      find_part_from_entns
       find_file
       find_product
       find_renderer
@@ -468,6 +720,8 @@ foreach
       ensure_parsed
 
       list_items
+      list_base
+      list_internal_base_folders
 
       add_to
     /
@@ -514,14 +768,34 @@ sub YATT::Lite::EntNS::entity_mkhidden {
   my ($this) = shift;
   \ join "\n", map {
     my $name = $_;
-    my $esc = escape($name);
     map {
-      sprintf(qq|<input type="hidden" name="%s" value="%s"/>|
-	      , $esc, escape($_));
-    } $CON->param($name);
-  } @_;
+      my $v = $_;
+      if (ref $v eq 'HASH') {
+        map {
+          _hidden_input(escape($name."[$_]"), $v->{$_});
+        } keys %$v;
+      } elsif (ref $v eq 'ARRAY') {
+        map {
+          _hidden_input(escape($name."[]"), $_);
+        } @$v;
+      } else {
+        _hidden_input(escape($name), $v);
+      }
+    } $CON->multi_param($name);
+  } do {
+    if (@_) {
+      @_;
+    } else {
+      $CON->param
+    }
+  };
 };
 
+sub _hidden_input {
+  sprintf(qq|<input type="hidden" name="%s" value="%s">|
+          , $_[0], escape($_[1]));
+}
+
 sub YATT::Lite::EntNS::entity_file_rootname {
   my ($this, $fn) = @_;
   $fn //= $CON->file();
@@ -529,6 +803,11 @@ sub YATT::Lite::EntNS::entity_file_rootname {
   $fn;
 };
 
+sub YATT::Lite::EntNS::entity_app_name {
+  my ($this) = @_;
+  $YATT->cget('app_name');
+};
+
 #----------------------------------------
 use YATT::Lite::Breakpoint ();
 YATT::Lite::Breakpoint::break_load_facade();
