@@ -12,6 +12,11 @@ my $dir = do {
   MY->rel2abs(untaint_any($d));
 };
 
+if (not -e "$FindBin::Bin/t_lib.pl" and not defined $_[0]) {
+  print STDERR "# Subdirectory tests should set \@_ to t_lib.pl\n";
+  exit 1;
+}
+
 use File::Basename;
 sub untaint_any {$_[0] =~ m{(.*)} and $1}
 use base qw/File::Spec/;
@@ -36,7 +41,7 @@ if (-d (my $d = "$dir/../blib/lib")."/YATT") {
 my $hook = sub {
   my ($this, $orig_modfn) = @_;
   return unless (my $modfn = $orig_modfn) =~ s!^YATT/!!;
-  Carp::cluck("orig_modfn=$orig_modfn\n") if $ENV{DEBUG_INC};
+  Carp::cluck("orig_modfn=$orig_modfn, dir=$dir, modfn=$modfn\n") if $ENV{DEBUG_INC};
   return unless -r (my $realfn = "$dir/../$modfn");
   warn "=> found $realfn" if $ENV{DEBUG_INC};
   open my $fh, '<', $realfn or die "Can't open $realfn:$!";
